`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : camera_ap_test.v
//-----------------------------------------------------------------------------
// Function             : camera sample application test
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2010/10/28
// -.-- 2010/11/17
//*****************************************************************************
module camera_ap_test;

        // Inputs
        reg clk;
        reg rstb;
        reg rxd;
        reg pclk;
        reg c_vsync;
        reg href;
        reg [7:0] in_data;

        // Outputs
        wire led_0;
        wire scl;
        wire txd;
        wire xclk;
        wire mon;
        wire da_en;
        wire [3:0] da_r;
        wire [3:0] da_g;
        wire [3:0] da_b;
        wire vsync;
        wire hsync;

        // Bidirs
        wire sda;

        // vsync,hsync
        reg [11:0] v_cnt;
        reg [11:0] h_cnt;

        // Instantiate the Unit Under Test (UUT)
        camera_ap uut (
                .clk(clk), 
                .rstb(rstb), 
                .led_0(led_0), 
                .scl(scl), 
                .sda(sda), 
                .txd(txd), 
                .rxd(rxd), 
                .xclk(xclk), 
                .pclk(pclk), 
                .c_vsync(c_vsync), 
                .href(href), 
                .in_data(in_data), 
                .mon(mon), 
                //.da_en(da_en), 
                .da_r(da_r), 
                .da_g(da_g), 
                .da_b(da_b), 
                .vsync(vsync), 
                .hsync(hsync)
        );

        initial begin
                // Initialize Inputs
                clk = 0;
                repeat(500000000) begin
                #12.5;
                clk = 1;
                #12.5;
                clk = 0;
                end
                $stop;
        end
        initial begin
                // Initialize Inputs
                pclk = 0;
                repeat(500000000) begin
                #12.5;
                pclk = 1;
                #12.5;
                pclk = 0;
                end
        end
           
        initial begin
                // Initialize Inputs
                rstb = 0;
                rxd = 1;

                // Wait 100 ns for global reset to finish
                #100;
                rstb = 1;
                // Add stimulus here
        end
      
        initial begin
                // Initialize Inputs
                rxd = 1;

                // Wait 100 ns for global reset to finish
                #190000;
                //#230000000;
                #8680;
                rxd = 1'b0;  //start #1 "W" 0x57
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b1;  // 2
                #8680;
                rxd = 1'b1;  // 3
                #8680;
                rxd = 1'b0;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b1;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #8680;
                #8680;
                rxd = 1'b0;  //start #2 "0" 0x31
                #8680;
                rxd = 1'b0;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b0;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b1;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #8680;
                #8680;
                #8680;
                rxd = 1'b0;  //start #3  "9" 0x39
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b1;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b1;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #4 "A" 0x41
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b0;  // 4
                #8680;
                rxd = 1'b0;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b1;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #5  "F" 0x46
                #8680;
                rxd = 1'b0;  // 1
                #8680;
                rxd = 1'b1;  // 2
                #8680;
                rxd = 1'b1;  // 3
                #8680;
                rxd = 1'b0;  // 4
                #8680;
                rxd = 1'b0;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b1;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #6  CR 0xd
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b1;  // 3
                #8680;
                rxd = 1'b1;  // 4
                #8680;
                rxd = 1'b0;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #6  LF 0xa
                #8680;
                rxd = 1'b0;  // 1
                #8680;
                rxd = 1'b1;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b1;  // 4
                #8680;
                rxd = 1'b0;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #8 "R" 0x52
                #8680;
                rxd = 1'b0;  // 1
                #8680;
                rxd = 1'b1;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b0;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b1;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #8680;
                #8680;
                rxd = 1'b0;  //start #9 "0" 0x31
                #8680;
                rxd = 1'b0;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b0;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b1;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #10  "9" 0x39
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b1;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b1;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #11 "0" 0x31
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b0;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b1;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #12  "0" 0x31
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b1;  // 4
                #8680;
                rxd = 1'b1;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b1;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #86800;
                rxd = 1'b0;  //start #13  CR 0xd
                #8680;
                rxd = 1'b1;  // 1
                #8680;
                rxd = 1'b0;  // 2
                #8680;
                rxd = 1'b1;  // 3
                #8680;
                rxd = 1'b1;  // 4
                #8680;
                rxd = 1'b0;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #86800;
                rxd = 1'b0;  //start #14  LF 0xa
                #8680;
                rxd = 1'b0;  // 1
                #8680;
                rxd = 1'b1;  // 2
                #8680;
                rxd = 1'b0;  // 3
                #8680;
                rxd = 1'b1;  // 4
                #8680;
                rxd = 1'b0;  // 5
                #8680;
                rxd = 1'b0;  // 6
                #8680;
                rxd = 1'b0;  // 7
                #8680;
                rxd = 1'b0;  // 8
                #8680;
                rxd = 1'b1;  // stop
                #8680;
                #8680;
                #86800;
                #86800;
                //$stop;
        
                // Add stimulus here

        end
      

always @ (posedge pclk or negedge rstb )
  if (rstb==1'b0)
    begin
      v_cnt <= 12'd0;
      h_cnt <= 12'd0;
    end
  else
    if (h_cnt == 12'd1567)
      begin
        h_cnt <= 12'd0;
        if (v_cnt == 12'd509)
          v_cnt <= 12'd0;
        else
          v_cnt <= v_cnt + 12'd1;
      end
    else
      begin
        v_cnt <= v_cnt ;
        h_cnt <= h_cnt + 12'd1;
      end // else: !if(h_cnt == 12'd1000)

always @ (posedge pclk or negedge rstb )
  if (rstb==1'b0)
    c_vsync <= 1'b0;
  else
    if ((v_cnt == 12'd0)&& (h_cnt == 12'd0))
      c_vsync <= 1'b1;
    else if ((v_cnt == 12'd2)&& (h_cnt == 12'd0))
      c_vsync <= 1'b0;
    else
      c_vsync <= c_vsync;

always @ (posedge pclk or negedge rstb )
  if (rstb==1'b0)
    href <= 1'b0;
  else
    if  ((v_cnt >= 12'd20)&& (v_cnt <= 12'd499))
      if (h_cnt == 12'd0)
        href <= 1'b1;
      else if (h_cnt == 12'd1290)
        href <= 1'b0;
      else
        href <= href;
    else
      href <= 1'b0;

always @ (posedge pclk or negedge rstb )
  if (rstb==1'b0)
    in_data <= 8'h00;
  else
    if (href == 1'b1)
      in_data <= in_data + 8'd1;
    else
      in_data <= in_data;
   
endmodule

